/* BdfServer - Copyright (c) 2006-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.EditSession;
import fr.exemole.bdfserver.api.roles.PermissionSummary;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.tools.configuration.ConfigurationUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.io.File;
import net.fichotheque.FichothequeEditor;
import net.fichotheque.corpus.CorpusEditor;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.Fiche;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.tools.corpus.CorpusTools;
import net.fichotheque.tools.corpus.FieldGenerationEngine;
import net.fichotheque.tools.parsers.CroisementParseEngine;
import net.fichotheque.tools.parsers.FicheParseEngine;
import net.fichotheque.tools.parsers.FicheParser;
import net.fichotheque.tools.parsers.ParseContext;
import net.fichotheque.tools.parsers.croisement.DocumentSubtokenParser;
import net.fichotheque.tools.parsers.croisement.IllustrationSubtokenParser;
import net.fichotheque.tools.parsers.croisement.SubtokenParameter;
import net.fichotheque.tools.parsers.croisement.SubtokenParameters;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.request.FileValue;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.request.RequestUtils;


/**
 *
 * @author Vincent Calame
 */
public final class EditionEngine {

    private final BdfServer bdfServer;
    private final ExtractionContext extractionContext;
    private final PermissionSummary permissionSummary;
    private final FichothequeEditor fichothequeEditor;
    private final RequestMap globalRequestMap;
    private final short ficheParseType;
    private final FicheParser.Parameters ficheParserParameters;
    private final Lang workingLang;
    private final ParseContext parseContext;

    private EditionEngine(EditSession editSession, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap, short ficheParseType) {
        this.bdfServer = editSession.getBdfServer();
        this.fichothequeEditor = editSession.getFichothequeEditor();
        this.globalRequestMap = requestMap;
        this.ficheParseType = ficheParseType;
        this.workingLang = bdfUser.getWorkingLang();
        this.ficheParserParameters = new FicheParser.Parameters(bdfUser, bdfServer.getL10nManager().getCodeCatalog(), BdfUserUtils.getContentChecker(bdfServer, bdfUser), BdfUserUtils.getTypoOptions(bdfUser));
        this.permissionSummary = permissionSummary;
        ficheParserParameters.setUserRedacteur(bdfUser.getRedacteur());
        this.extractionContext = BdfServerUtils.initExtractionContextBuilder(bdfServer, bdfUser)
                .setSubsetAccessPredicate(permissionSummary.getSubsetAccessPredicate())
                .toExtractionContext();
        this.parseContext = new ParseContext(fichothequeEditor, bdfServer.getPolicyManager().getDynamicEditPolicies(), bdfServer.getThesaurusLangChecker(), bdfServer.getExternalSourceProvider(), extractionContext);
    }

    /**
     * Séparateur des champs « en ligne » (Rédacteurs, Information). Par défaut
     * : ';'
     */
    public EditionEngine setItemListSeparateur(char itemListSeparateur) {
        ficheParserParameters.setItemListSeparateur(itemListSeparateur);
        return this;
    }

    public EditionEngine updateFiche(FicheMeta ficheMeta) {
        updateFiche(ficheMeta, null);
        return this;
    }

    public EditionEngine updateFiche(FicheMeta ficheMeta, @Nullable String paramSuffix) {
        RequestMap requestMap;
        if (paramSuffix != null) {
            requestMap = RequestUtils.reduceToSuffix(globalRequestMap, paramSuffix + "_");
        } else {
            requestMap = globalRequestMap;
        }
        CroisementParseEngine.run(ficheMeta, requestMap, parseContext, workingLang);
        SubtokenParameters subtokenParameters = SubtokenParameters.build(requestMap.getParameterNameSet());
        if (subtokenParameters.withIllustration()) {
            File tmpDirectory = ConfigurationUtils.getTmpDirectory(bdfServer);
            IllustrationSubtokenParser illustrationSubtokenParser = new IllustrationSubtokenParser(ficheMeta, tmpDirectory);
            for (SubtokenParameter subtokenParameter : subtokenParameters.getIllustrationFileList()) {
                FileValue[] fileValues = requestMap.getFileValues(subtokenParameter.getParamName());
                if (fileValues != null) {
                    illustrationSubtokenParser.addFromMultipart(subtokenParameter, fileValues);
                }
            }
            for (SubtokenParameter subtokenParameter : subtokenParameters.getIllustrationList()) {
                illustrationSubtokenParser.parse(subtokenParameter, requestMap.getParameterValues(subtokenParameter.getParamName()));
            }
            illustrationSubtokenParser.save(fichothequeEditor);
        }
        if (subtokenParameters.withDocument()) {
            File tmpDirectory = ConfigurationUtils.getTmpDirectory(bdfServer);
            DocumentSubtokenParser documentSubtokenParser = new DocumentSubtokenParser(ficheMeta, tmpDirectory);
            for (SubtokenParameter subtokenParameter : subtokenParameters.getDocumentFileList()) {
                FileValue[] fileValues = requestMap.getFileValues(subtokenParameter.getParamName());
                if (fileValues != null) {
                    documentSubtokenParser.addFromMultipart(subtokenParameter, fileValues);
                }
            }
            for (SubtokenParameter subtokenParameter : subtokenParameters.getDocumentList()) {
                documentSubtokenParser.parse(subtokenParameter, requestMap.getParameterValues(subtokenParameter.getParamName()));
            }
            documentSubtokenParser.save(fichothequeEditor);
        }
        if (ficheParseType != FicheParseEngine.NONE_TYPE) {
            editFiche(ficheMeta, requestMap);
        }
        return this;
    }

    private void editFiche(FicheMeta ficheMeta, RequestMap requestMap) {
        Fiche fiche = FicheParseEngine.run(ficheParserParameters, ficheMeta, requestMap, ficheParseType);
        CorpusEditor corpusEditor = fichothequeEditor.getCorpusEditor(ficheMeta.getCorpus().getSubsetKey());
        Lang defaultLang = ConfigurationUtils.getDefaultWorkingLang(bdfServer.getLangConfiguration());
        FieldGenerationEngine engine = FieldGenerationEngine.build(ficheMeta.getCorpus(), extractionContext, bdfServer.getFormatContext(), defaultLang);
        CorpusTools.saveFiche(corpusEditor, ficheMeta, fiche, engine, true);
    }

    public final static EditionEngine build(EditSession editSession, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap, short ficheParseType) {
        return new EditionEngine(editSession, bdfUser, permissionSummary, requestMap, ficheParseType);
    }

    public final static EditionEngine replace(EditSession editSession, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap) {
        return build(editSession, bdfUser, permissionSummary, requestMap, FicheParseEngine.REPLACE_TYPE);
    }

    public final static EditionEngine update(EditSession editSession, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap) {
        return build(editSession, bdfUser, permissionSummary, requestMap, FicheParseEngine.UPDATE_TYPE);
    }

    public final static EditionEngine croisementOnly(EditSession editSession, BdfUser bdfUser, PermissionSummary permissionSummary, RequestMap requestMap) {
        return build(editSession, bdfUser, permissionSummary, requestMap, FicheParseEngine.NONE_TYPE);
    }


}
