/* BdfServer - Copyright (c) 2011-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.balayage;

import fr.exemole.bdfserver.api.balayage.BalayageConstants;
import fr.exemole.bdfserver.api.balayage.BalayageUnit;
import fr.exemole.bdfserver.api.balayage.Output;
import java.util.ArrayList;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleQuery;
import net.fichotheque.utils.SelectionUtils;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public class BalayageUnitBuilder implements BalayageConstants {

    private final Set<String> modeSet = new LinkedHashSet<String>();
    private final Set<String> extensionSet = new LinkedHashSet<String>();
    private final List<OutputBuilder> outputList = new ArrayList<OutputBuilder>();
    private final short type;
    private String name;
    private String extractionPath;
    private boolean globalSelect = true;
    private FicheQuery ficheQuery = SelectionUtils.EMPTY_FICHEQUERY;
    private MotcleQuery motcleQuery = SelectionUtils.EMPTY_MOTCLEQUERY;
    private IllustrationQuery illustrationQuery = SelectionUtils.EMPTY_ILLUSTRATIONQUERY;
    private DocumentQuery documentQuery = SelectionUtils.EMPTY_DOCUMENTQUERY;

    public BalayageUnitBuilder(short type) {
        this.type = type;
    }

    public void setName(String name) {
        this.name = name;
    }

    public void setGlobalSelect(boolean globalSelect) {
        this.globalSelect = globalSelect;
    }

    public void setExtractionPath(String extractionPath) {
        this.extractionPath = extractionPath;
    }

    public void addMode(String mode) {
        modeSet.add(mode);
    }

    public void setFicheQuery(@Nullable FicheQuery ficheQuery) {
        if (ficheQuery == null) {
            this.ficheQuery = SelectionUtils.EMPTY_FICHEQUERY;
        } else {
            this.ficheQuery = ficheQuery;
        }
    }

    public void setMotcleQuery(@Nullable MotcleQuery motcleQuery) {
        if (motcleQuery == null) {
            this.motcleQuery = SelectionUtils.EMPTY_MOTCLEQUERY;
        } else {
            this.motcleQuery = motcleQuery;
        }
    }

    public void setIllustrationQuery(@Nullable IllustrationQuery illustrationQuery) {
        if (illustrationQuery == null) {
            this.illustrationQuery = SelectionUtils.EMPTY_ILLUSTRATIONQUERY;
        } else {
            this.illustrationQuery = illustrationQuery;
        }
    }

    public void setDocumentQuery(@Nullable DocumentQuery documentQuery) {
        if (documentQuery == null) {
            this.documentQuery = SelectionUtils.EMPTY_DOCUMENTQUERY;
        } else {
            this.documentQuery = documentQuery;
        }
    }

    public void addExtension(String s) {
        extensionSet.add(s);
    }

    public OutputBuilder addOutputBuilder() {
        OutputBuilder iob = new OutputBuilder();
        outputList.add(iob);
        return iob;
    }

    public BalayageUnit toBalayageUnit() {
        int size = modeSet.size();
        String[] modeArray = null;
        if (size > 0) {
            modeArray = new String[size];
            int p = 0;
            for (String mode : modeSet) {
                modeArray[p] = mode;
                p++;
            }
        }
        List<String> extensionList;
        if (extensionSet.isEmpty()) {
            extensionList = StringUtils.EMPTY_STRINGLIST;
        } else {
            extensionList = StringUtils.wrap(extensionSet.toArray(new String[extensionSet.size()]));
        }
        int size3 = outputList.size();
        Output[] outputArray = null;
        if (size3 > 0) {
            outputArray = new Output[size3];
        }
        BalayageUnit balayageUnit = new InternalBalayageUnit(type, name, extractionPath, globalSelect, ficheQuery, motcleQuery, illustrationQuery, documentQuery, outputArray, modeArray, extensionList);
        if (size3 > 0) {
            for (int i = 0; i < size3; i++) {
                outputArray[i] = outputList.get(i).toOutput(balayageUnit);
            }
        }
        return balayageUnit;
    }


    private static class InternalBalayageUnit implements BalayageUnit {

        private final short type;
        private final String name;
        private final String extractionPath;
        private boolean globalSelect = true;
        private final FicheQuery ficheQuery;
        private final MotcleQuery motcleQuery;
        private final IllustrationQuery illustrationQuery;
        private final DocumentQuery documentQuery;
        private final Output[] outputArray;
        private final String[] modeArray;
        private final List<String> extensionList;


        private InternalBalayageUnit(short type, String name, String extractionPath, boolean globalSelect,
                FicheQuery ficheQuery, MotcleQuery motcleQuery, IllustrationQuery illustrationQuery, DocumentQuery documentQuery,
                Output[] outputArray, String[] modeArray, List<String> extensionList) {
            this.type = type;
            this.name = name;
            this.extractionPath = extractionPath;
            this.globalSelect = globalSelect;
            this.ficheQuery = ficheQuery;
            this.motcleQuery = motcleQuery;
            this.illustrationQuery = illustrationQuery;
            this.documentQuery = documentQuery;
            this.outputArray = outputArray;
            this.modeArray = modeArray;
            this.extensionList = extensionList;
        }

        @Override
        public String getName() {
            return name;
        }

        @Override
        public short getType() {
            return type;
        }

        @Override
        public boolean isGlobalSelect() {
            return globalSelect;
        }

        @Override
        public FicheQuery getFicheQuery() {
            return ficheQuery;
        }

        @Override
        public MotcleQuery getMotcleQuery() {
            return motcleQuery;
        }

        @Override
        public IllustrationQuery getIllustrationQuery() {
            return illustrationQuery;
        }

        @Override
        public DocumentQuery getDocumentQuery() {
            return documentQuery;
        }

        @Override
        public int getOutputCount() {
            if (outputArray == null) {
                return 0;
            }
            return outputArray.length;
        }

        @Override
        public Output getOutput(int i) {
            return outputArray[i];
        }

        @Override
        public boolean hasMode(String s) {
            if (modeArray == null) {
                return false;
            }
            int size = modeArray.length;
            if (size == 0) {
                return false;
            }
            for (int i = 0; i < size; i++) {
                if (modeArray[i].equals(s)) {
                    return true;
                }
            }
            return false;
        }

        @Override
        public List<String> getExtensionList() {
            return extensionList;
        }

        @Override
        public String getExtractionPath() {
            return extractionPath;
        }

    }

}
