/* BdfServer - Copyright (c) 2006-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.balayage.engine;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.balayage.Balayage;
import fr.exemole.bdfserver.api.balayage.BalayageMode;
import fr.exemole.bdfserver.api.balayage.SiteMapOption;
import fr.exemole.bdfserver.api.configuration.PathConfiguration;
import fr.exemole.bdfserver.tools.BdfURI;
import java.io.File;
import net.mapeadores.util.text.RelativePath;


/**
 *
 * @author Vincent Calame
 */
public class BalayageInitializer {

    private final BdfServer bdfServer;
    private final Balayage balayage;
    private final PathConfiguration pathConfiguration;
    private File balayageRootDirectory;
    private String errorMessageKey;
    private Object errorMessageValue;
    private BalayageMode balayageMode;

    public BalayageInitializer(BdfServer bdfServer, Balayage balayage, PathConfiguration pathConfiguration) {
        this.bdfServer = bdfServer;
        this.balayage = balayage;
        this.pathConfiguration = pathConfiguration;
        init();
    }

    private void init() {
        String rootName = balayage.getRootName();
        if (rootName == null) {
            errorMessageKey = "_ error.empty.balayage.root";
            return;
        }
        File rootDirectory = pathConfiguration.getTargetDirectory(rootName);
        if (rootDirectory == null) {
            rootDirectory = bdfServer.getOutputStorage().getFile(RelativePath.build("balayages/" + balayage.getName()));
            rootDirectory.mkdirs();
        }
        if (!rootDirectory.exists()) {
            errorMessageKey = "_ error.unknown.balayage.root";
            String[] err = {balayage.getRootName(), rootDirectory.getAbsolutePath()};
            errorMessageValue = err;
            return;
        }
        if (!rootDirectory.isDirectory()) {
            errorMessageKey = "_ error.wrong.balayage.root_notdirectory";
            String[] err = {balayage.getRootName(), rootDirectory.getAbsolutePath()};
            errorMessageValue = err;
            return;
        }
        String path = balayage.getPath();
        if (path != null) {
            if (!pathTest(path)) {
                errorMessageKey = "_ error.wrong.balayage.relativepath";
                errorMessageValue = path;
                return;
            }
            balayageRootDirectory = new File(rootDirectory, path);
            if (!balayageRootDirectory.exists()) {
                balayageRootDirectory.mkdirs();
            } else if (!balayageRootDirectory.isDirectory()) {
                errorMessageKey = "_ error.wrong.balayage.path_notdirectory";
                errorMessageValue = balayageRootDirectory.getAbsolutePath();
                return;
            }
        } else {
            balayageRootDirectory = rootDirectory;
        }
        if (balayage.getBalayageUnitCount() == 0) {
            errorMessageKey = "_ error.empty.balayage.unit";
        }
    }

    public boolean hasError() {
        return (errorMessageKey != null);
    }

    public String getErrorMessageKey() {
        return errorMessageKey;
    }

    public Object getErrorMessageValue() {
        return errorMessageValue;
    }

    public void setBalayageMode(BalayageMode balayageMode) {
        this.balayageMode = balayageMode;
    }

    public BalayageEngine getBalayageEngine(BalayageLog balayageLog) {
        if (hasError()) {
            return null;
        }
        BalayageParameters balayageParameters = new BalayageParameters(balayage, bdfServer, pathConfiguration);
        balayageParameters.setBalayageMode(balayageMode);
        balayageParameters.setBalayageLog(balayageLog);
        balayageParameters.setBalayageRootDirectory(balayageRootDirectory);
        balayageParameters.setCustomURIResolver(BdfURI.getURIResolver(bdfServer, pathConfiguration, true));
        SiteMapOption siteMapOption = balayage.getSiteMapOption();
        if (siteMapOption != null) {
            SiteMapBuilder siteMapBuilder = new SiteMapBuilder(siteMapOption);
            balayageParameters.setFileGenerationListener(siteMapBuilder);
        }
        BalayageEngine balayageEngine = new BalayageEngine(balayageParameters);
        return balayageEngine;
    }

    static private boolean pathTest(String path) {
        return (!path.contains(".."));
    }

}
