/* BdfServer - Copyright (c) 2009-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.ficheform.builders;

import fr.exemole.bdfserver.api.ficheform.FicheFormParameters;
import fr.exemole.bdfserver.api.ficheform.RattachementElement;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.tools.L10nUtils;
import java.util.List;
import net.fichotheque.Subset;
import net.fichotheque.SubsetItem;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.utils.CorpusUtils;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;


/**
 *
 * @author Vincent Calame
 */
public class RattachementElementBuilder {

    private final String name;
    private String label = "";
    private boolean mandatory = false;
    private SubsetItem masterSubsetItem;
    private List<FicheMeta> satelliteFicheMetaList;
    private Attributes attributes = AttributeUtils.EMPTY_ATTRIBUTES;

    public RattachementElementBuilder(String name) {
        this.name = name;
    }

    public RattachementElementBuilder setLabel(String label) {
        if (label == null) {
            throw new IllegalArgumentException("label is null");
        }
        this.label = label;
        return this;
    }

    public RattachementElementBuilder setMandatory(boolean mandatory) {
        this.mandatory = mandatory;
        return this;
    }

    public RattachementElementBuilder setAttributes(Attributes attributes) {
        if (attributes == null) {
            this.attributes = AttributeUtils.EMPTY_ATTRIBUTES;
        } else {
            this.attributes = attributes;
        }
        return this;
    }

    public RattachementElementBuilder setMasterSubsetItem(SubsetItem masterSubsetItem) {
        this.masterSubsetItem = masterSubsetItem;
        return this;
    }

    public RattachementElementBuilder setSatelliteFicheMetaList(List<FicheMeta> satelliteFicheMetaList) {
        this.satelliteFicheMetaList = satelliteFicheMetaList;
        return this;
    }

    public RattachementElement toRattachementElement() {
        return new InternalRattachementElement(name, label, mandatory, attributes, masterSubsetItem, satelliteFicheMetaList);
    }

    @Nullable
    public static RattachementElementBuilder check(FicheFormParameters ficheFormParameters, IncludeUi includeUi) {
        FichePointeur fichePointeur = ficheFormParameters.getFichePointeur();
        RattachementElementBuilder builder = init(includeUi.getName())
                .setAttributes(includeUi.getAttributes());
        Subset masterSubset = ((Corpus) fichePointeur.getSubset()).getMasterSubset();
        SubsetItem currentSubsetItem = fichePointeur.getCurrentSubsetItem();
        if (currentSubsetItem == null) {
            if (masterSubset != null) {
                SubsetItem masterSubsetItem = ficheFormParameters.getMasterSubsetItem();
                if (masterSubsetItem == null) {
                    return null;
                } else {
                    builder.setMasterSubsetItem(masterSubsetItem);
                }
            } else {
                builder.setMasterSubsetItem(null);
            }
            builder.setSatelliteFicheMetaList(CorpusUtils.EMPTY_FICHEMETALIST);
        } else {
            if (masterSubset == null) {
                builder
                        .setMasterSubsetItem(null)
                        .setSatelliteFicheMetaList(CorpusUtils.getSatelliteFicheMetaList(currentSubsetItem));
            } else {
                builder
                        .setMasterSubsetItem(masterSubset.getSubsetItemById(currentSubsetItem.getId()))
                        .setSatelliteFicheMetaList(CorpusUtils.getOtherSatelliteFicheMetaList((FicheMeta) currentSubsetItem));
            }
        }
        return builder
                .setLabel(L10nUtils.toLabelString(ficheFormParameters, includeUi));
    }

    public static RattachementElementBuilder init(String name) {
        return new RattachementElementBuilder(name);
    }


    private static class InternalRattachementElement implements RattachementElement {

        private final String name;
        private final String label;
        private final boolean mandatory;
        private final Attributes attributes;
        private final SubsetItem masterSubsetItem;
        private final List<FicheMeta> satelliteFicheMetaList;

        private InternalRattachementElement(String name, String label, boolean mandatory, Attributes attributes, SubsetItem masterSubsetItem, List<FicheMeta> satelliteFicheMetaList) {
            this.name = name;
            this.label = label;
            this.mandatory = mandatory;
            this.attributes = attributes;
            this.masterSubsetItem = masterSubsetItem;
            this.satelliteFicheMetaList = satelliteFicheMetaList;
        }

        @Override
        public String getIncludeName() {
            return name;
        }

        @Override
        public String getLabel() {
            return label;
        }

        @Override
        public boolean isMandatory() {
            return mandatory;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

        @Override
        public SubsetItem getMasterSubsetItem() {
            return masterSubsetItem;
        }

        @Override
        public List<FicheMeta> getSatelliteFicheMetaList() {
            return satelliteFicheMetaList;
        }

    }

}
