/* BdfServer - Copyright (c) 2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.tools.uml;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.ui.IncludeUi;
import fr.exemole.bdfserver.api.ui.UiComponent;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;
import net.fichotheque.Fichotheque;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.include.ExtendedIncludeKey;
import net.fichotheque.utils.FichothequeUtils;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.MessageLocalisation;
import net.mapeadores.util.uml.PumlUtils;


/**
 *
 * @author Vincent Calame
 */
public class CorpusPumlWriter {

    private final static int SATELLITE = 1;
    private final static int MASTER = 2;
    private final BdfServer bdfServer;
    private final Lang lang;
    private final MessageLocalisation messageLocalisation;
    private final Corpus corpus;
    private final SortedMap<SubsetKey, LinkInfo> infoMap = new TreeMap<SubsetKey, LinkInfo>();
    private final List<LinkInfo> thesaurusList = new ArrayList<LinkInfo>();
    private final List<LinkInfo> corpusList = new ArrayList<LinkInfo>();

    public CorpusPumlWriter(BdfServer bdfServer, Lang lang, MessageLocalisation messageLocalisation, Corpus corpus) {
        this.bdfServer = bdfServer;
        this.lang = lang;
        this.messageLocalisation = messageLocalisation;
        this.corpus = corpus;
        scanLinkInfo();
    }

    public void write(Appendable appendable) throws IOException {
        SubsetKey corpusKey = corpus.getSubsetKey();
        PumlUtils.start(appendable);
        PumlUtils.title(appendable, FichothequeUtils.getTitle(corpus, lang));
        PumlUtils.hideEmpties(appendable);
        PumlUtils.orthoLineType(appendable);
        BdfPumlUtils.buildCorpusUmlClass(corpus, lang, messageLocalisation).write(appendable, 0);
        for (LinkInfo linkInfo : infoMap.values()) {
            Subset subset = linkInfo.subset;
            if (!subset.getSubsetKey().equals(corpusKey)) {
                BdfPumlUtils.buildUmlClass(subset, lang).write(appendable, 0);
            }
        }
        appendable.append("\n");
        for (LinkInfo linkInfo : infoMap.values()) {
            linkInfo.write(appendable, corpusKey);
        }
        int thesaurusSize = thesaurusList.size();
        if (thesaurusSize > 0) {
            int corpusSize = corpusList.size();
            for (int i = 0; i < corpusSize; i++) {
                LinkInfo corpusInfo = corpusList.get(i);
                int thesaurusIndex = Math.min(i, (thesaurusSize - 1));
                LinkInfo thesaurusInfo = thesaurusList.get(thesaurusIndex);
                appendable.append(thesaurusInfo.getSubsetKey().toString());
                appendable.append(" -[hidden]- ");
                appendable.append(corpusInfo.getSubsetKey().toString());
                appendable.append("\n");
            }
        }
        PumlUtils.end(appendable);
    }

    private void scanLinkInfo() {
        Subset masterSubset = corpus.getMasterSubset();
        if (masterSubset != null) {
            LinkInfo linkInfo = new LinkInfo(masterSubset, MASTER);
            infoMap.put(masterSubset.getSubsetKey(), linkInfo);
        }
        for (Corpus satelliteCorpus : corpus.getSatelliteCorpusList()) {
            LinkInfo linkInfo = new LinkInfo(satelliteCorpus, SATELLITE);
            infoMap.put(satelliteCorpus.getSubsetKey(), linkInfo);
        }
        Fichotheque fichotheque = bdfServer.getFichotheque();
        for (UiComponent uiComponent : bdfServer.getUiManager().getMainUiComponents(corpus).getUiComponentList()) {
            if (uiComponent instanceof IncludeUi) {
                IncludeUi includeUi = (IncludeUi) uiComponent;
                ExtendedIncludeKey eik = includeUi.getExtendedIncludeKey();
                if (eik != null) {
                    SubsetKey otherKey = eik.getSubsetKey();
                    if (eik.isMaster()) {

                    } else {
                        String mode = eik.getMode();
                        LinkInfo linkInfo = infoMap.get(otherKey);
                        if (linkInfo == null) {
                            Subset otherSubset = fichotheque.getSubset(otherKey);
                            if (otherSubset != null) {
                                linkInfo = new LinkInfo(otherSubset);
                                linkInfo.addMode(mode);
                            }
                            infoMap.put(otherKey, linkInfo);
                        } else {
                            linkInfo.addMode(mode);
                        }
                    }
                }
            }
        }
        for (LinkInfo linkInfo : infoMap.values()) {
            if (!linkInfo.isSpecial()) {
                switch (linkInfo.getSubsetKey().getCategory()) {
                    case SubsetKey.CATEGORY_CORPUS:
                        corpusList.add(linkInfo);
                        break;
                    case SubsetKey.CATEGORY_THESAURUS:
                        thesaurusList.add(linkInfo);
                        break;
                }
            }
        }
    }

    public static String toString(BdfServer bdfServer, Lang lang, MessageLocalisation messageLocalisation, Corpus corpus) {
        CorpusPumlWriter corpusPumlWriter = new CorpusPumlWriter(bdfServer, lang, messageLocalisation, corpus);
        try {
            StringBuilder buf = new StringBuilder();
            corpusPumlWriter.write(buf);
            return buf.toString();
        } catch (IOException ioe) {
            throw new ShouldNotOccurException();
        }
    }


    private static class LinkInfo {

        private final Subset subset;
        private final SortedSet<String> modeSet = new TreeSet<String>();
        private final int special;

        private LinkInfo(Subset subset) {
            this.subset = subset;
            this.special = 0;
        }

        private LinkInfo(Subset subset, int special) {
            this.subset = subset;
            this.special = special;
        }

        private boolean isSpecial() {
            return (special != 0);
        }

        private void addMode(String mode) {
            modeSet.add(mode);
        }

        private SubsetKey getSubsetKey() {
            return subset.getSubsetKey();
        }

        private void write(Appendable appendable, SubsetKey corpusKey) throws IOException {
            SubsetKey otherSubsetKey = subset.getSubsetKey();
            if (special != 0) {
                appendable.append(corpusKey.toString());
                appendable.append(getRelation(special));
                appendable.append(otherSubsetKey.toString());
                appendable.append("\n");
            }
            short otherSubsetCategory = otherSubsetKey.getCategory();
            for (String mode : modeSet) {
                appendable.append(corpusKey.toString());
                if (special != 0) {
                    if (otherSubsetCategory == SubsetKey.CATEGORY_THESAURUS) {
                        appendable.append(" <-down- ");
                    } else {
                        appendable.append(getCorpusRelation(special));
                    }
                } else {
                    appendable.append(BdfPumlUtils.getRelation(otherSubsetCategory));
                }
                appendable.append(otherSubsetKey.toString());
                if (mode.length() > 0) {
                    appendable.append(" : ");
                    appendable.append(mode);
                }
                appendable.append("\n");
            }
        }

    }

    private static String getRelation(int special) {
        switch (special) {
            case SATELLITE:
                return " *-up- ";
            case MASTER:
                return " -down-* ";
            default:
                return " -- ";
        }
    }

    private static String getCorpusRelation(int special) {
        switch (special) {
            case SATELLITE:
                return " -up- ";
            case MASTER:
                return " -down- ";
            default:
                return " -- ";
        }
    }

}
