/* BdfServer_HtmlProducers - Copyright (c) 2012-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.edition;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.ficheform.FicheForm;
import fr.exemole.bdfserver.api.instruction.BdfCommandResult;
import fr.exemole.bdfserver.api.instruction.BdfInstructionConstants;
import fr.exemole.bdfserver.api.instruction.BdfInstructionException;
import fr.exemole.bdfserver.api.instruction.OutputParameters;
import fr.exemole.bdfserver.api.instruction.ProducerParameters;
import fr.exemole.bdfserver.api.interaction.domains.EditionDomain;
import fr.exemole.bdfserver.api.managers.UiManager;
import fr.exemole.bdfserver.api.ui.UiComponents;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.tools.ficheform.FicheFormBuilder;
import fr.exemole.bdfserver.tools.ficheform.FicheFormParametersBuilder;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.instruction.PermissionChecker;
import fr.exemole.bdfserver.tools.ui.UiUtils;
import net.fichotheque.SubsetItem;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.include.LiageTest;
import net.fichotheque.pointeurs.FichePointeur;
import net.fichotheque.utils.pointeurs.PointeurFactory;
import net.mapeadores.util.html.HtmlProducer;
import net.mapeadores.util.request.RequestMap;


/**
 *
 * @author Vincent Calame
 */
public final class EditionHtmlProducerFactory {

    private final static String ALTER_PARAMNAME = "alter";
    private final static String PAGE_RESULT_PARAMNAME = "page-result";

    private EditionHtmlProducerFactory() {
    }

    public static HtmlProducer getHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        BdfServerHtmlProducer htmlProducer = getBdfHtmlProducer(parameters);
        if (htmlProducer != null) {
            htmlProducer.setBdfCommandResult(parameters.getBdfCommandResult());
        }
        return htmlProducer;
    }

    private static BdfServerHtmlProducer getBdfHtmlProducer(OutputParameters parameters) throws BdfInstructionException {
        String page = parameters.getOutput();
        BdfServer bdfServer = parameters.getBdfServer();
        RequestMap requestMap = parameters.getRequestMap();
        BdfUser bdfUser = parameters.getBdfUser();
        BdfCommandResult bdfCommandResult = parameters.getBdfCommandResult();
        PermissionChecker checker = new PermissionChecker(bdfServer, bdfUser);
        switch (page) {
            case EditionDomain.ZOOMEDIT_PAGE: {
                String appelant = BdfInstructionUtils.getMandatoryParameter(requestMap, "appelant");
                return new ZoomEditHtmlProducer(checker, appelant);
            }
            case EditionDomain.FICHE_RESULT_PAGE: {
                FicheMeta ficheMeta = getFicheMeta(bdfServer, requestMap, bdfCommandResult, bdfUser);
                return new FicheResultHtmlProducer(checker, ficheMeta);
            }
            case EditionDomain.FICHE_INDEXATION_PAGE: {
                FicheMeta ficheMeta = getFicheMeta(bdfServer, requestMap, bdfCommandResult, bdfUser);
                checker.checkSubsetAdmin(ficheMeta.getCorpus());
                FicheForm ficheForm = getFicheForm(checker, ficheMeta);
                return new FicheIndexationHtmlProducer(checker, ficheMeta, ficheForm);
            }
            case EditionDomain.FICHE_CREATION_PAGE: {
                Corpus corpus = getCorpus(bdfServer, requestMap, bdfCommandResult);
                checker.checkFicheCreate(corpus);
                if (corpus.getMasterSubset() != null) {
                    return new SatelliteFicheCreationHtmlProducer(checker, corpus);
                } else {
                    UiComponents uiComponents = getUiComponents(bdfServer, requestMap, corpus);
                    FichePointeur fichePointeur = PointeurFactory.newFichePointeur(corpus, true);
                    FicheFormParametersBuilder ficheFormParametersBuilder = initFicheFormParametersBuilder(checker, requestMap, fichePointeur)
                            .setNewFiche(true);
                    FicheForm ficheForm = FicheFormBuilder.toFicheForm(ficheFormParametersBuilder.toFicheFormParameters());
                    FicheFormHtmlProducer ficheFormHtmlProducer = FicheFormHtmlProducer.newCreationInstance(checker, corpus, ficheForm, uiComponents);
                    String pageResult = requestMap.getParameter(PAGE_RESULT_PARAMNAME);
                    if (pageResult != null) {
                        ficheFormHtmlProducer.setFicheResultPage(pageResult);
                    }
                    return ficheFormHtmlProducer;
                }
            }
            case EditionDomain.FICHE_CHANGE_PAGE: {
                Corpus corpus = getCorpus(bdfServer, requestMap, bdfCommandResult);
                UiComponents uiComponents = getUiComponents(bdfServer, requestMap, corpus);
                FicheMeta ficheMeta = null;
                FichePointeur fichePointeur = PointeurFactory.newFichePointeur(corpus, true);
                FicheFormParametersBuilder ficheFormParametersBuilder = initFicheFormParametersBuilder(checker, requestMap, fichePointeur);
                int id = -1;
                if (bdfCommandResult != null) {
                    ficheMeta = (FicheMeta) bdfCommandResult.getResultObject(BdfInstructionConstants.FICHEMETA_OBJ);
                }
                if (ficheMeta == null) {
                    id = BdfInstructionUtils.getMandatoryId(requestMap);
                    ficheMeta = corpus.getFicheMetaById(id);
                }
                if (ficheMeta != null) {
                    checker.checkWrite(ficheMeta);
                    fichePointeur.setCurrentSubsetItem(ficheMeta);
                    ficheFormParametersBuilder.setNewFiche(false);
                    FicheForm ficheForm = FicheFormBuilder.toFicheForm(ficheFormParametersBuilder.toFicheFormParameters());
                    return FicheFormHtmlProducer.newChangeInstance(checker, ficheMeta, ficheForm, uiComponents);
                } else {
                    checker.checkFicheCreate(corpus);
                    SubsetItem masterSubsetItem = BdfInstructionUtils.checkMasterSubsetItem(corpus, id, bdfUser);
                    ficheFormParametersBuilder.setMasterSubsetItem(masterSubsetItem);
                    boolean force = requestMap.isTrue(EditionDomain.FORCE_PARAMNAME);
                    if (!force) {
                        return new CreationConfirmHtmlProducer(checker, corpus, id, masterSubsetItem);
                    }
                    ficheFormParametersBuilder.setNewFiche(true);
                    FicheForm ficheForm = FicheFormBuilder.toFicheForm(ficheFormParametersBuilder.toFicheFormParameters());
                    FicheFormHtmlProducer ficheFormHtmlProducer;
                    if (masterSubsetItem != null) {
                        ficheFormHtmlProducer = FicheFormHtmlProducer.newCreationInstance(checker, corpus, masterSubsetItem, ficheForm, uiComponents);
                    } else {
                        ficheFormHtmlProducer = FicheFormHtmlProducer.newCreationInstance(checker, corpus, id, ficheForm, uiComponents);
                    }
                    String pageResult = requestMap.getParameter(PAGE_RESULT_PARAMNAME);
                    if (pageResult != null) {
                        ficheFormHtmlProducer.setFicheResultPage(pageResult);
                    }
                    return ficheFormHtmlProducer;
                }
            }
            default:
                return null;
        }
    }

    public static FicheForm getFicheForm(ProducerParameters producerParameters, FicheMeta ficheMeta) {
        FichePointeur fichePointeur = toFichePointeur(ficheMeta);
        return FicheFormBuilder.toFicheForm(FicheFormParametersBuilder.build(producerParameters, fichePointeur)
                .setNewFiche(false)
                .toFicheFormParameters());
    }

    private static FichePointeur toFichePointeur(FicheMeta ficheMeta) {
        FichePointeur fichePointeur = PointeurFactory.newFichePointeur(ficheMeta.getCorpus());
        fichePointeur.setCurrentSubsetItem(ficheMeta);
        return fichePointeur;
    }

    public static Corpus getCorpus(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult) throws BdfInstructionException {
        Corpus corpus = null;
        if (bdfCommandResult != null) {
            corpus = (Corpus) bdfCommandResult.getResultObject(BdfInstructionConstants.CORPUS_OBJ);
            if (corpus == null) {
                FicheMeta ficheMeta = (FicheMeta) bdfCommandResult.getResultObject(BdfInstructionConstants.FICHEMETA_OBJ);
                if (ficheMeta != null) {
                    corpus = ficheMeta.getCorpus();
                }
            }
        }
        if (corpus == null) {
            corpus = BdfInstructionUtils.getMandatoryCorpus(bdfServer.getFichotheque(), requestMap);
        }
        return corpus;
    }

    public static FicheMeta getFicheMeta(BdfServer bdfServer, RequestMap requestMap, BdfCommandResult bdfCommandResult, BdfUser bdfUser) throws BdfInstructionException {
        FicheMeta ficheMeta = null;
        if (bdfCommandResult != null) {
            ficheMeta = (FicheMeta) bdfCommandResult.getResultObject(BdfInstructionConstants.FICHEMETA_OBJ);
        }
        if (ficheMeta == null) {
            ficheMeta = BdfInstructionUtils.getMandatoryFicheMeta(bdfServer.getFichotheque(), requestMap, bdfUser);
        }
        return ficheMeta;
    }

    public static FicheFormParametersBuilder initFicheFormParametersBuilder(ProducerParameters producerParameters, RequestMap requestMap, FichePointeur fichePointeur) {
        boolean withSpecialParseChars;
        if (requestMap.isTrue("brackets")) {
            withSpecialParseChars = false;
        } else {
            withSpecialParseChars = true;
        }
        LiageTest liageTest = UiUtils.checkLiageTest(producerParameters.getBdfServer().getUiManager().getMainUiComponents((Corpus) fichePointeur.getSubset()));
        return FicheFormParametersBuilder.build(producerParameters, fichePointeur)
                .initSupplementaryParameter(requestMap)
                .setLiageTest(liageTest)
                .setWithSpecialParseChars(withSpecialParseChars);
    }

    public static UiComponents getUiComponents(BdfServer bdfServer, RequestMap requestMap, Corpus corpus) {
        UiManager uiManager = bdfServer.getUiManager();
        String alterName = requestMap.getParameter(ALTER_PARAMNAME);
        //return uiManager.getComponentUiHolder(corpus, alterName);
        return uiManager.getMainUiComponents(corpus);
    }

}
