/* BdfServer_HtmlProducers - Copyright (c) 2013-2021 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.htmlproducers.main;

import fr.exemole.bdfserver.api.BdfServer;
import fr.exemole.bdfserver.api.interaction.Domains;
import fr.exemole.bdfserver.api.interaction.GetConstants;
import fr.exemole.bdfserver.api.interaction.InteractionConstants;
import fr.exemole.bdfserver.api.interaction.domains.MailingDomain;
import fr.exemole.bdfserver.api.interaction.domains.MainDomain;
import fr.exemole.bdfserver.api.users.BdfUser;
import fr.exemole.bdfserver.html.BdfHtmlConstants;
import fr.exemole.bdfserver.html.BdfServerHtmlProducer;
import fr.exemole.bdfserver.html.consumers.Button;
import fr.exemole.bdfserver.html.consumers.Tree;
import fr.exemole.bdfserver.tools.BH;
import fr.exemole.bdfserver.tools.BdfHref;
import fr.exemole.bdfserver.tools.exportation.table.BdfTableExportUtils;
import fr.exemole.bdfserver.tools.instruction.BdfInstructionUtils;
import fr.exemole.bdfserver.tools.users.BdfUserUtils;
import java.util.List;
import net.fichotheque.Subset;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.exportation.table.TableExportDescription;
import net.fichotheque.thesaurus.Thesaurus;
import net.mapeadores.util.html.HA;
import net.mapeadores.util.html.HtmlAttributes;
import net.mapeadores.util.html.HtmlPrinter;
import net.mapeadores.util.html.HtmlWrapper;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;


/**
 *
 * @author Vincent Calame
 */
public final class MainHtmlUtils {

    public final static HtmlWrapper DISPLAYTABLE_TREE = Tree.tree("displaytable-Tree");
    public final static HtmlWrapper DISPLAYTABLE_LEAF = Tree.leaf("displaytable-Leaf");

    private MainHtmlUtils() {
    }

    public static String getMainPageName(BdfServerHtmlProducer bhp) {
        if (bhp.isWithJavascript()) {
            return BdfInstructionUtils.getAbsolutePageName(Domains.MAIN, MainDomain.IFRAMES_PAGE);
        } else {
            return BdfInstructionUtils.getAbsolutePageName(Domains.MAIN, MainDomain.FRAMESET_PAGE);
        }
    }

    public static boolean printTableDisplayDetails(BdfServerHtmlProducer hp, BdfServer bdfServer, BdfUser bdfUser, Subset subset, boolean withFicheMailing, HtmlAttributes detailsAttributes) {
        SubsetKey subsetKey = subset.getSubsetKey();
        List<TableExportDescription> list = BdfTableExportUtils.getAvailableTableExportDescriptionList(bdfServer.getTableExportManager(), subset.getSubsetKey());
        String currentTableExportName = BdfUserUtils.getSubsetTableExportName(bdfUser, subset);
        TableExportDescription currentTableExportDescription = null;
        if (currentTableExportName != null) {
            for (TableExportDescription tableExportDescription : list) {
                String name = tableExportDescription.getName();
                if (name.equals(currentTableExportName)) {
                    currentTableExportDescription = tableExportDescription;
                    break;
                }
            }
            if (currentTableExportDescription == null) {
                currentTableExportName = null;
            }
        }
        String titleKey = "_ link.main.tabledisplay";
        if (subset instanceof Corpus) {
            titleKey = "_ link.main.tabledisplay_fiches";
        } else if (subset instanceof Thesaurus) {
            titleKey = "_ link.main.tabledisplay_motscles";
        }
        Lang workingLang = bdfUser.getWorkingLang();
        hp
                .DETAILS(detailsAttributes)
                .SUMMARY()
                .__localize(titleKey)
                ._SUMMARY()
                .__(DISPLAYTABLE_TREE, () -> {
                    for (TableExportDescription tableExportDescription : list) {
                        String name = tableExportDescription.getName();
                        hp
                                .__(DISPLAYTABLE_LEAF, () -> {
                                    hp
                                            .SPAN("displaytable-Title")
                                            .__escape(getTitle(tableExportDescription, workingLang))
                                            .__colon()
                                            ._SPAN()
                                            .__(printSupplementaryLinks(hp, subsetKey, name, withFicheMailing));
                                });
                    }
                    hp
                            .__(DISPLAYTABLE_LEAF, () -> {
                                hp
                                        .SPAN("displaytable-Title displaytable-Default")
                                        .__localize("_ link.main.defaulttableformat")
                                        .__colon()
                                        ._SPAN()
                                        .__(printSupplementaryLinks(hp, subsetKey, null, withFicheMailing));
                            });
                })
                ._DETAILS();

        return true;
    }

    public static String getTitle(TableExportDescription tableExportDescription, Lang lang) {
        Label label = LabelUtils.getLangPartCheckedLabel(tableExportDescription.getTableExportDef().getTitleLabels(), lang);
        if (label != null) {
            return label.getLabelString();
        } else {
            return tableExportDescription.getName();
        }
    }

    private static boolean printSupplementaryLinks(HtmlPrinter hp, SubsetKey subsetKey, String tableExportName, boolean withMailing) {
        BdfHref displayHref = BH.domain(Domains.MAIN).page(MainDomain.TABLEDISPLAY_PAGE).param(InteractionConstants.SUBSET_PARAMNAME, subsetKey.toString()).param(MainDomain.REMEMBER_ACTION_PARAMNAME, "1");
        String subsetKeyString = subsetKey.toString();
        String urlPrefix = GetConstants.TABLES_ROOT + "/" + subsetKeyString;
        if (tableExportName != null) {
            urlPrefix = urlPrefix + "-" + tableExportName;
            displayHref.param(InteractionConstants.TABLEEXPORT_PARAMNAME, tableExportName);
        }
        hp
                .A(HA.href(displayHref).classes("global-button-Transparent action-TableDisplay").titleMessage("_ link.global.version", "HTML").target(BdfHtmlConstants.EDITION_FRAME))
                .__(Button.ICON)
                ._A()
                .A(HA.href(urlPrefix + ".csv").classes("global-button-Transparent action-Csv").titleMessage("_ link.global.version", "CSV"))
                .__(Button.ICON)
                ._A()
                .A(HA.href(urlPrefix + ".ods").classes("global-button-Transparent action-Ods").titleMessage("_ link.global.version", "ODS"))
                .__(Button.ICON)
                ._A();
        if (withMailing) {
            BdfHref sendHref = BH.domain(Domains.MAILING).page(MailingDomain.SEND_FORM_PAGE).subset(subsetKey).param("type", MailingDomain.TABLEEXPORT_SENDTYPE);
            if (tableExportName != null) {
                sendHref.tableExport(tableExportName);
            }
            hp
                    .A(HA.href(sendHref).target(BdfHtmlConstants.EDITION_FRAME).classes("global-button-Transparent action-Send").titleLocKey("_ link.global.send"))
                    .__(Button.ICON)
                    ._A();
        }
        return true;
    }

}
