/* FichothequeLib_API - Copyright (c) 2007-2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils;

import java.text.ParseException;
import java.util.AbstractList;
import java.util.List;
import java.util.RandomAccess;
import net.fichotheque.Fichotheque;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.FicheAPI;
import net.fichotheque.corpus.fiche.FicheItem;
import net.fichotheque.corpus.fiche.Information;
import net.fichotheque.corpus.fiche.Personne;
import net.fichotheque.corpus.fiche.Propriete;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.sphere.Sphere;
import net.mapeadores.util.annotation.Nullable;
import net.mapeadores.util.models.PersonCore;
import net.mapeadores.util.models.PersonCoreUtils;


/**
 *
 * @author Vincent Calame
 */
public final class SphereUtils {

    public final static short MALFORMED_KEY = 1;
    public final static short UNKNOWN_SPHERE = 2;
    public final static short UNKNOWN_LOGIN = 3;

    private SphereUtils() {
    }

    /**
     * Un identifiant de connexion (login) doit répondre à l'expression
     * rationnelle : [a-zA-Z][a-zA-Z0-9]*.
     */
    public static void checkLogin(String login) throws ParseException {
        int length = login.length();
        if (length == 0) {
            throw new ParseException("empty login", 0);
        }
        if (!testFirstChar(login.charAt(0))) {
            throw new ParseException("wrong first char", 0);
        }
        for (int i = 1; i < length; i++) {
            if (!testChar(login.charAt(i))) {
                throw new ParseException("wrong char", i);
            }
        }
    }

    private static boolean testFirstChar(char carac) {
        if ((carac >= 'A') && (carac <= 'Z')) {
            return true;
        } else if ((carac >= 'a') && (carac <= 'z')) {
            return true;
        } else {
            return false;
        }
    }

    private static boolean testChar(char carac) {
        if ((carac >= 'A') && (carac <= 'Z')) {
            return true;
        } else if ((carac >= 'a') && (carac <= 'z')) {
            return true;
        } else if ((carac >= '0') && (carac <= '9')) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Un identifiant de connexion (login) doit répondre à l'expression
     * rationnelle : [a-zA-Z][a-zA-Z0-9]*.
     */
    public static boolean testLogin(String login) {
        try {
            checkLogin(login);
            return true;
        } catch (ParseException pe) {
            return false;
        }
    }

    public static Redacteur getRedacteur(Fichotheque fichotheque, String redacteurGlobalId) {
        try {
            return (Redacteur) FichothequeUtils.parseGlobalId(redacteurGlobalId, fichotheque, SubsetKey.CATEGORY_SPHERE, null);
        } catch (ParseException pe) {
            return null;
        }
    }

    public static PersonCore toPersonCore(Fichotheque fichotheque, Personne personne) {
        String redacteurGlobalId = personne.getRedacteurGlobalId();
        if (redacteurGlobalId != null) {
            Redacteur redacteur = getRedacteur(fichotheque, redacteurGlobalId);
            if (redacteur != null) {
                return redacteur.getPersonCore();
            } else {
                return PersonCoreUtils.toPersonCore(redacteurGlobalId, "", "", false);
            }
        } else {
            return personne.getPersonCore();
        }
    }

    public static boolean testRedacteurs(Corpus corpus, Redacteur redacteur) {
        String globalId = redacteur.getGlobalId();
        for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
            if (SphereUtils.ownsToRedacteur(ficheMeta, globalId)) {
                return true;
            }
        }
        return false;
    }

    public static boolean testRedacteursAndFields(Corpus corpus, Redacteur redacteur, List<CorpusField> personneList) {
        String globalId = redacteur.getGlobalId();
        int size = personneList.size();
        for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
            if (SphereUtils.ownsToRedacteur(ficheMeta, globalId)) {
                return true;
            }
            if (size > 1) {
                FicheAPI fiche = corpus.getEnteteFiche(ficheMeta);
                for (int j = 1; j < size; j++) {
                    CorpusField corpusField = personneList.get(j);
                    FieldKey fieldKey = corpusField.getFieldKey();
                    if (fieldKey.isProprieteFieldKey()) {
                        Propriete propriete = fiche.getPropriete(fieldKey);
                        if (propriete != null) {
                            boolean here = testFicheItem(redacteur, propriete.getFicheItem());
                            if (here) {
                                return true;
                            }
                        }
                    } else if (fieldKey.isInformationFieldKey()) {
                        Information information = fiche.getInformation(fieldKey);
                        if (information != null) {
                            int size3 = information.size();
                            for (int k = 0; k < size3; k++) {
                                boolean here = testFicheItem(redacteur, information.get(k));
                                if (here) {
                                    return true;
                                }
                            }
                        }
                    }
                }
            }
        }
        return false;
    }

    public static boolean testFicheItem(Redacteur redacteur, FicheItem ficheItem) {
        if (!(ficheItem instanceof Personne)) {
            return false;
        }
        String redacteurGlobalId = ((Personne) ficheItem).getRedacteurGlobalId();
        if (redacteurGlobalId != null) {
            return redacteurGlobalId.equals(redacteur.getGlobalId());
        } else {
            return false;
        }
    }

    public static String toBracketStyle(Redacteur redacteur) {
        return redacteur.getLogin() + "[" + redacteur.getSubset().getSubsetKey().getSubsetName() + "]";
    }

    public static String toCompleteName(Redacteur redacteur) {
        return redacteur.getPersonCore().toStandardStyle();
    }

    public static boolean testGlobalId(String redacteurGlobalId) {
        try {
            getSubsetKey(redacteurGlobalId);
            getId(redacteurGlobalId);
            return true;
        } catch (ParseException pe) {
            return false;
        }
    }

    public static SubsetKey getSubsetKey(String redacteurGlobalId) throws ParseException {
        int idx = redacteurGlobalId.indexOf('/');
        if (idx == -1) {
            throw new ParseException("Missing /", 0);
        }
        return SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, redacteurGlobalId.substring(0, idx));
    }

    public static int getId(String redacteurGlobalId) throws ParseException {
        int idx = redacteurGlobalId.indexOf('/');
        if (idx == -1) {
            throw new ParseException("Missing /", 0);
        }
        try {
            int id = Integer.parseInt(redacteurGlobalId.substring(idx + 1));
            if (idx < 1) {
                throw new ParseException("Not positive integer", idx);
            }
            return id;
        } catch (NumberFormatException nfe) {
            throw new ParseException("Not integer", idx);
        }
    }

    public static boolean ownsToRedacteur(FicheMeta ficheMeta, Redacteur redacteur) {
        return ownsToRedacteur(ficheMeta, redacteur.getGlobalId());
    }

    public static boolean ownsToRedacteur(FicheMeta ficheMeta, String redacteurGlobalId) {
        return ficheMeta.getRedacteurGlobalIdList().contains(redacteurGlobalId);
    }

    public static boolean ownsToSphere(FicheMeta ficheMeta, Sphere sphere) {
        return ownsToSphere(ficheMeta, sphere.getSubsetKey().getSubsetName());
    }

    public static boolean ownsToSphere(FicheMeta ficheMeta, String sphereName) {
        List<String> globalIdList = ficheMeta.getRedacteurGlobalIdList();
        if (globalIdList.isEmpty()) {
            return false;
        }
        String prefix = sphereName + "/";
        for (String redacteurGlobalId : globalIdList) {
            if (redacteurGlobalId.startsWith(prefix)) {
                return true;
            }
        }
        return false;
    }

    public static Redacteur parse(Fichotheque fichotheque, String key) throws RedacteurLoginException {
        return parse(fichotheque, key, null);
    }

    public static Redacteur parse(Fichotheque fichotheque, String key, @Nullable SubsetKey defaultSphereKey) throws RedacteurLoginException {
        int idx = key.indexOf('[');
        if (idx != -1) {
            return parseBrackets(fichotheque, key);
        }
        idx = key.indexOf('_');
        if (idx != -1) {
            return parseUnderscore(fichotheque, key);
        }
        idx = key.indexOf('/');
        if (idx != -1) {
            Redacteur redacteur = getRedacteur(fichotheque, key);
            if (redacteur != null) {
                return redacteur;
            } else {
                throw new RedacteurLoginException(MALFORMED_KEY, "Wrong redacteurGlobalId");
            }
        }
        if (defaultSphereKey == null) {
            throw new RedacteurLoginException(MALFORMED_KEY, "Missing separator (_ or [])");
        } else {
            Sphere defaultSphere = (Sphere) fichotheque.getSubset(defaultSphereKey);
            if (defaultSphere == null) {
                throw new RedacteurLoginException(UNKNOWN_SPHERE, "unknown default sphere: " + defaultSphereKey.toString());
            } else {
                Redacteur redacteur = defaultSphere.getRedacteurByLogin(key);
                if (redacteur == null) {
                    throw new RedacteurLoginException(UNKNOWN_LOGIN, "unknown redacteur: " + key);
                }
                return redacteur;
            }
        }
    }

    private static Redacteur parseBrackets(Fichotheque fichotheque, String key) throws RedacteurLoginException {
        int idx1 = key.indexOf('[');
        int idx2 = key.indexOf(']');
        if (idx2 < 0) {
            throw new RedacteurLoginException(MALFORMED_KEY, "Missing ]");
        }
        if (idx2 < idx1) {
            throw new RedacteurLoginException(MALFORMED_KEY, "] before [");
        }
        if (idx2 < (key.length() - 1)) {
            throw new RedacteurLoginException(MALFORMED_KEY, "characters after ]");
        }
        String login = key.substring(0, idx1).trim();
        String sphereString = key.substring(idx1 + 1, idx2).trim();
        SubsetKey sphereKey;
        try {
            sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereString);
        } catch (ParseException pe) {
            throw new RedacteurLoginException(MALFORMED_KEY, "malformed SubsetKey: " + sphereString);
        }
        Sphere sphere = (Sphere) fichotheque.getSubset(sphereKey);
        if (sphere == null) {
            throw new RedacteurLoginException(UNKNOWN_SPHERE, "unknown sphere: " + sphereString);
        }
        Redacteur redacteur = sphere.getRedacteurByLogin(login);
        if (redacteur == null) {
            throw new RedacteurLoginException(UNKNOWN_LOGIN, "unknown redacteur: " + login);
        }
        return redacteur;
    }

    private static Redacteur parseUnderscore(Fichotheque fichotheque, String key) throws RedacteurLoginException {
        int idx = key.indexOf('_');
        if (idx == 0) {
            throw new RedacteurLoginException(MALFORMED_KEY, "Starting with _");
        }
        if (idx == (key.length() - 1)) {
            throw new RedacteurLoginException(MALFORMED_KEY, "Ending with _");
        }
        String login = key.substring(idx + 1);
        String sphereString = key.substring(0, idx);
        SubsetKey sphereKey;
        try {
            sphereKey = SubsetKey.parse(SubsetKey.CATEGORY_SPHERE, sphereString);
        } catch (ParseException pe) {
            throw new RedacteurLoginException(MALFORMED_KEY, "malformed SubsetKey: " + sphereString);
        }
        Sphere sphere = (Sphere) fichotheque.getSubset(sphereKey);
        if (sphere == null) {
            throw new RedacteurLoginException(UNKNOWN_SPHERE, "unknown sphere: " + sphereString);
        }
        Redacteur redacteur = sphere.getRedacteurByLogin(login);
        if (redacteur == null) {
            throw new RedacteurLoginException(UNKNOWN_LOGIN, "unknown redacteur: " + login);
        }
        return redacteur;
    }

    public static List<Redacteur> wrap(Redacteur[] array) {
        return new RedacteurList(array);
    }


    public static class RedacteurLoginException extends Exception {

        private final short type;

        private RedacteurLoginException(short type, String message) {
            super(message);
            this.type = type;
        }

        public short getType() {
            return type;
        }

    }


    private static class RedacteurList extends AbstractList<Redacteur> implements RandomAccess {

        private final Redacteur[] array;

        private RedacteurList(Redacteur[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public Redacteur get(int index) {
            return array[index];
        }

    }

}
