/* FichothequeLib_Tools - Copyright (c) 2011-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.extraction.builders;

import java.util.ArrayList;
import java.util.List;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.MotcleFilter;


/**
 *
 * @author Vincent Calame
 */
public class MotcleFilterBuilder {

    private final List<ExtraitCorpusDef> indexationExtraitCorpusDefList = new ArrayList<ExtraitCorpusDef>();
    private final List<FicheRattachementDef> ficheRattachementDefList = new ArrayList<FicheRattachementDef>();
    private String tagName = ExtractionConstants.MOTCLE_DEFAULT_TAGNAME;
    private boolean withLevel = false;
    private boolean withLabels = false;
    private boolean withIntitule = false;
    private MotcleFilter previousFilter = null;
    private MotcleFilter nextFilter = null;
    private boolean parentRecursive = false;
    private MotcleFilter parentFilter = null;
    private boolean childrenRecursive = false;
    private MotcleFilter childrenFilter = null;
    private boolean withThesaurusTitle = false;


    public MotcleFilterBuilder() {
    }

    public MotcleFilterBuilder setTagName(String tagName) {
        this.tagName = BuilderUtils.testTagName(tagName, ExtractionConstants.MOTCLE_DEFAULT_TAGNAME);
        return this;
    }

    public MotcleFilterBuilder setTagName(String tagName, String defaultTagName) {
        this.tagName = BuilderUtils.testTagName(tagName, defaultTagName);
        return this;
    }

    public MotcleFilterBuilder setWithLevel(boolean withLevel) {
        this.withLevel = withLevel;
        return this;
    }

    public MotcleFilterBuilder setWithIntitule(boolean withIntitule) {
        this.withIntitule = withIntitule;
        return this;
    }

    public MotcleFilterBuilder setChildrenFilter(MotcleFilter childrenFilter) {
        this.childrenRecursive = false;
        this.childrenFilter = childrenFilter;
        return this;
    }

    public MotcleFilterBuilder setParentFilter(MotcleFilter parentFilter) {
        this.parentRecursive = false;
        this.parentFilter = parentFilter;
        return this;
    }

    public MotcleFilterBuilder setNextFilter(MotcleFilter nextFilter) {
        this.nextFilter = nextFilter;
        return this;
    }

    public MotcleFilterBuilder setWithLabels(boolean withLabels) {
        this.withLabels = withLabels;
        return this;
    }

    public MotcleFilterBuilder setPreviousFilter(MotcleFilter previousFilter) {
        this.previousFilter = previousFilter;
        return this;
    }

    public MotcleFilterBuilder setWithThesaurusTitle(boolean withThesaurusTitle) {
        this.withThesaurusTitle = withThesaurusTitle;
        return this;
    }

    public MotcleFilterBuilder addDefaultIndexationExtraitCorpusDef() {
        return addIndexationExtraitCorpusDef(BuilderUtils.getExtraitCorpusDef(FicheFilterBuilder.TITRE_FICHEFILTER, IntituleClauseBuilder.DEFAULT));
    }

    public MotcleFilterBuilder addIndexationExtraitCorpusDef(ExtraitCorpusDef indexationExtraitCorpusDef) {
        if (indexationExtraitCorpusDef == null) {
            throw new IllegalArgumentException("indexationExtraitCorpusDef is null");
        }
        indexationExtraitCorpusDefList.add(indexationExtraitCorpusDef);
        return this;
    }

    public MotcleFilterBuilder setChildrenRecursive() {
        this.childrenRecursive = true;
        this.childrenFilter = null;
        return this;
    }

    public MotcleFilterBuilder setParentRecursive() {
        this.parentRecursive = true;
        this.parentFilter = null;
        return this;
    }

    public MotcleFilterBuilder addFicheRattachementDef(FicheRattachementDef ficheRattachementDef) {
        if (ficheRattachementDef == null) {
            throw new IllegalArgumentException("ficheRattachementDef is null");
        }
        ficheRattachementDefList.add(ficheRattachementDef);
        return this;
    }

    public MotcleFilter toMotcleFilter() {
        List<ExtraitCorpusDef> finalIndexationExtraitCorpusDefList;
        int indexationSize = indexationExtraitCorpusDefList.size();
        if (indexationSize == 0) {
            finalIndexationExtraitCorpusDefList = BuilderUtils.EMPTY_EXTRAITCORPUSDEFLIST;
        } else {
            finalIndexationExtraitCorpusDefList = BuilderUtils.wrap(indexationExtraitCorpusDefList.toArray(new ExtraitCorpusDef[indexationSize]));
        }
        int rattachementSize = ficheRattachementDefList.size();
        List<FicheRattachementDef> finalFicheRattachementDefList;
        if (rattachementSize == 0) {
            finalFicheRattachementDefList = BuilderUtils.EMPTY_FICHERATTACHEMENTDEFLIST;
        } else {
            finalFicheRattachementDefList = BuilderUtils.wrap(ficheRattachementDefList.toArray(new FicheRattachementDef[rattachementSize]));
        }
        return new InternalMotcleFilter(tagName, finalIndexationExtraitCorpusDefList, withLevel, withLabels, withIntitule,
                previousFilter, nextFilter, parentRecursive, parentFilter, childrenRecursive, childrenFilter, withThesaurusTitle, finalFicheRattachementDefList);
    }

    public static MotcleFilterBuilder init() {
        return new MotcleFilterBuilder();
    }


    private static class InternalMotcleFilter implements MotcleFilter {

        private final String tagName;
        private final List<ExtraitCorpusDef> indexationExtraitCorpusDefList;
        private final boolean withLevel;
        private final boolean withLabels;
        private final boolean withIntitule;
        private final MotcleFilter previousFilter;
        private final MotcleFilter nextFilter;
        private final boolean parentRecursive;
        private final MotcleFilter parentFilter;
        private final boolean childrenRecursive;
        private final MotcleFilter childrenFilter;
        private final boolean withThesaurusTitle;
        private final List<FicheRattachementDef> ficheRattachementDefList;

        private InternalMotcleFilter(String tagName, List<ExtraitCorpusDef> indexationExtraitCorpusDefList, boolean withLevel, boolean withLabels, boolean withIntitule,
                MotcleFilter previousFilter, MotcleFilter nextFilter, boolean parentRecursive, MotcleFilter parentFilter, boolean childrenRecursive, MotcleFilter childrenFilter,
                boolean withThesaurusTitle, List<FicheRattachementDef> ficheRattachementDefList) {
            this.tagName = tagName;
            this.indexationExtraitCorpusDefList = indexationExtraitCorpusDefList;
            this.withLevel = withLevel;
            this.withLabels = withLabels;
            this.withIntitule = withIntitule;
            this.previousFilter = previousFilter;
            this.nextFilter = nextFilter;
            this.parentRecursive = parentRecursive;
            this.parentFilter = parentFilter;
            this.childrenRecursive = childrenRecursive;
            this.childrenFilter = childrenFilter;
            this.withThesaurusTitle = withThesaurusTitle;
            this.ficheRattachementDefList = ficheRattachementDefList;
        }

        @Override
        public boolean isNoneFiltering() {
            if (withLabels) {
                return false;
            }
            if (!indexationExtraitCorpusDefList.isEmpty()) {
                return false;
            }
            if (withLevel) {
                return false;
            }
            if (withIntitule) {
                return false;
            }
            if (previousFilter != null) {
                return false;
            }
            if (nextFilter != null) {
                return false;
            }
            if ((parentRecursive) || (parentFilter != null)) {
                return false;
            }
            if ((childrenRecursive) || (childrenFilter != null)) {
                return false;
            }
            if (!ficheRattachementDefList.isEmpty()) {
                return false;
            }
            return true;
        }

        @Override
        public boolean withLevel() {
            return withLevel;
        }

        @Override
        public boolean withLabels() {
            return withLabels;
        }

        @Override
        public boolean withIntitule() {
            return withIntitule;
        }

        @Override
        public MotcleFilter getChildrenFilter() {
            if (childrenRecursive) {
                return this;
            } else {
                return childrenFilter;
            }
        }

        @Override
        public MotcleFilter getParentFilter() {
            if (parentRecursive) {
                return this;
            } else {
                return parentFilter;
            }
        }

        @Override
        public MotcleFilter getNextFilter() {
            return nextFilter;
        }

        @Override
        public MotcleFilter getPreviousFilter() {
            return previousFilter;
        }

        @Override
        public boolean isWithThesaurusTitle() {
            return withThesaurusTitle;
        }

        @Override
        public List<ExtraitCorpusDef> getIndexationExtraitCorpusDefList() {
            return indexationExtraitCorpusDefList;
        }

        @Override
        public List<FicheRattachementDef> getFicheRattachementDefList() {
            return ficheRattachementDefList;
        }

        @Override
        public String getTagName() {
            return tagName;
        }

    }

}
