/* FichothequeLib_Tools - Copyright (c) 2006-2017 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.tools.sphere;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedMap;
import java.util.TreeMap;
import net.fichotheque.Fichotheque;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.corpus.fiche.FicheAPI;
import net.fichotheque.corpus.fiche.Information;
import net.fichotheque.corpus.fiche.Propriete;
import net.fichotheque.corpus.metadata.CorpusField;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.sphere.Redacteur;
import net.fichotheque.utils.CorpusMetadataUtils;
import net.fichotheque.utils.SphereUtils;


/**
 *
 * @author Vincent Calame
 */
public class RedacteurStats {

    private final Redacteur redacteur;
    private final List<ByCorpusStats> byCorpusStatsList = new ArrayList<ByCorpusStats>();
    private int count = 0;

    public RedacteurStats(Redacteur redacteur) {
        this.redacteur = redacteur;
        init();
    }

    public boolean isEmpty() {
        return byCorpusStatsList.isEmpty();
    }

    public int getFicheCount() {
        return count;
    }

    private void init() {
        Fichotheque fichotheque = redacteur.getSphere().getFichotheque();
        for (Corpus corpus : fichotheque.getCorpusList()) {
            List<CorpusField> personneList = CorpusMetadataUtils.getCorpusFieldListByFicheItemType(corpus.getCorpusMetadata(), CorpusField.PERSONNE_FIELD);
            ByCorpusStats byCorpusStats;
            if (personneList.size() == 1) {
                byCorpusStats = initRedacteur(corpus);
            } else {
                byCorpusStats = initRedacteurAndFields(corpus, personneList);
            }
            if (byCorpusStats != null) {
                byCorpusStatsList.add(byCorpusStats);
                count = count + byCorpusStats.getFicheCount();
            }
        }
    }

    private ByCorpusStats initRedacteur(Corpus corpus) {
        ByCorpusStats byCorpusStats = null;
        String globalId = redacteur.getGlobalId();
        for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
            if (SphereUtils.ownsToRedacteur(ficheMeta, globalId)) {
                if (byCorpusStats == null) {
                    byCorpusStats = new ByCorpusStats(corpus);
                }
                byCorpusStats.add(ficheMeta.getId(), 1);
            }
        }
        return byCorpusStats;
    }

    private ByCorpusStats initRedacteurAndFields(Corpus corpus, List<CorpusField> personneList) {
        String globalId = redacteur.getGlobalId();
        ByCorpusStats byCorpusStats = null;
        int size = personneList.size();
        for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
            int poids = 0;
            if (SphereUtils.ownsToRedacteur(ficheMeta, globalId)) {
                poids = 1;
            }
            if (size > 1) {
                FicheAPI fiche = corpus.getEnteteFiche(ficheMeta);
                for (int j = 1; j < size; j++) {
                    CorpusField corpusField = personneList.get(j);
                    FieldKey fieldKey = corpusField.getFieldKey();
                    if (fieldKey.isProprieteFieldKey()) {
                        Propriete propriete = fiche.getPropriete(fieldKey);
                        if (propriete != null) {
                            boolean here = SphereUtils.testFicheItem(redacteur, propriete.getFicheItem());
                            if (here) {
                                poids = poids | 2;
                            }
                        }
                    } else if (fieldKey.isInformationFieldKey()) {
                        Information information = fiche.getInformation(fieldKey);
                        if (information != null) {
                            int size3 = information.size();
                            for (int k = 0; k < size3; k++) {
                                boolean here = SphereUtils.testFicheItem(redacteur, information.get(k));
                                if (here) {
                                    poids = poids | 4;
                                    break;
                                }
                            }
                        }
                    }
                }
                if (poids > 0) {
                    if (byCorpusStats == null) {
                        byCorpusStats = new ByCorpusStats(corpus);
                    }
                    byCorpusStats.add(ficheMeta.getId(), poids);
                }
            }
        }
        return byCorpusStats;
    }


    public static class ByCorpusStats {

        private final Corpus corpus;
        private final SortedMap<Integer, ByFicheStats> treeMap = new TreeMap<Integer, ByFicheStats>();

        ByCorpusStats(Corpus corpus) {
            this.corpus = corpus;
        }

        public int getFicheCount() {
            return treeMap.size();
        }

        public Corpus getCorpus() {
            return corpus;
        }

        private void add(int id, int poids) {
            ByFicheStats byFicheStats = new ByFicheStats(id, poids);
            treeMap.put(id, byFicheStats);
        }

        public List<ByFicheStats> getByFicheStatsList() {
            return new ArrayList<ByFicheStats>(treeMap.values());
        }

    }


    public static class ByFicheStats {

        private final int id;
        private final int poids;

        ByFicheStats(int id, int poids) {
            this.id = id;
            this.poids = poids;
        }

        public int getId() {
            return id;
        }

        public int getPoids() {
            return poids;
        }

    }

    public List<ByCorpusStats> getByCorpusStatsList() {
        return byCorpusStatsList;
    }

}
