/* FichothequeLib_Xml - Copyright (c) 2013-2019 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml;

import java.io.IOException;
import java.util.List;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.extraction.ExtractionConstants;
import net.fichotheque.extraction.def.DocumentFilter;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.ExtraitAddendaDef;
import net.fichotheque.extraction.def.ExtraitAlbumDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.FicheRattachementDef;
import net.fichotheque.extraction.def.GroupClause;
import net.fichotheque.extraction.def.GroupParams;
import net.fichotheque.extraction.def.IllustrationFilter;
import net.fichotheque.extraction.def.IntituleClause;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.MotcleRattachementDef;
import net.fichotheque.extraction.def.PackClause;
import net.fichotheque.extraction.filterunit.ChronoFilterUnit;
import net.fichotheque.extraction.filterunit.CorpsdeficheFilterUnit;
import net.fichotheque.extraction.filterunit.EnteteFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitAddendaFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitAlbumFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitCorpusFilterUnit;
import net.fichotheque.extraction.filterunit.ExtraitThesaurusFilterUnit;
import net.fichotheque.extraction.filterunit.FicheRattachementFilterUnit;
import net.fichotheque.extraction.filterunit.FieldKeyFilterUnit;
import net.fichotheque.extraction.filterunit.FieldNamePrefixFilterUnit;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.extraction.filterunit.IntituleFilterUnit;
import net.fichotheque.extraction.filterunit.LotFilterUnit;
import net.fichotheque.extraction.filterunit.MotcleRattachementFilterUnit;
import net.fichotheque.namespaces.BdfXmlSpace;
import net.fichotheque.xml.extraction.ExtractionXMLUtils;
import net.mapeadores.util.attr.Attribute;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.attr.Attributes;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import net.fichotheque.selection.DocumentQuery;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.IllustrationQuery;
import net.fichotheque.selection.MotcleQuery;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionDefXMLPart extends XMLPart {

    public ExtractionDefXMLPart(XMLWriter xmlWriter) {
        super(xmlWriter);
    }

    public void appendExtractionDef(ExtractionDef extractionDef) throws IOException {
        startOpenTag("extraction");
        addTagName(extractionDef.getTagName(), ExtractionConstants.EXTRACTION_DEFAULT_TAGNAME);
        endOpenTag();
        addExtraitCorpus(extractionDef.getDynamicExtraitCorpusDef());
        addExtraitThesaurus(extractionDef.getDynamicExtraitThesaurusDef());
        List<ExtraitCorpusDef> corpusDefList = extractionDef.getStaticExtraitCorpusDefList();
        List<ExtraitThesaurusDef> thesaurusDefList = extractionDef.getStaticExtraitThesaurusDefList();
        if ((!corpusDefList.isEmpty()) || (!thesaurusDefList.isEmpty())) {
            startOpenTag("static");
            addTagName(extractionDef.getStaticTagName(), ExtractionConstants.STATIC_DEFAULT_TAGNAME);
            endOpenTag();
            for (ExtraitCorpusDef extraitCorpusDef : corpusDefList) {
                addExtraitCorpus(extraitCorpusDef);
            }
            for (ExtraitThesaurusDef extraitThesaurusDef : thesaurusDefList) {
                addExtraitThesaurus(extraitThesaurusDef);
            }
            closeTag("static");
        }
        closeTag("extraction");
    }

    private void addExtraitCorpus(ExtraitCorpusDef extraitCorpusDef) throws IOException {
        if (extraitCorpusDef == null) {
            return;
        }
        startOpenTag("extraitcorpus");
        addAttribute("extrait-name", extraitCorpusDef.getExtraitName());
        addTagName(extraitCorpusDef.getTagName(), ExtractionConstants.EXTRAITCORPUS_DEFAULT_TAGNAME);
        if (extraitCorpusDef.isMaster()) {
            addAttribute("source", "master");
        }
        endOpenTag();
        for (FicheQuery ficheQuery : extraitCorpusDef.getFicheQueryList()) {
            ExtractionXMLUtils.writeFicheQuery(this, ficheQuery, "fiche-select");
        }
        addGroupClause(extraitCorpusDef.getGroupClause());
        addPackClause(extraitCorpusDef.getPackClause());
        addIntituleClause(extraitCorpusDef.getIntituleClause());
        FicheFilter ficheFilter = extraitCorpusDef.getFicheFilter();
        startOpenTag("fiche");
        addTagName(ficheFilter.getTagName(), ExtractionConstants.FICHE_DEFAULT_TAGNAME);
        List<FilterUnit> filterUnitList = ficheFilter.getFilterUnitList();
        if (filterUnitList.isEmpty()) {
            closeEmptyTag();
        } else {
            endOpenTag();
            for (FilterUnit filterUnit : filterUnitList) {
                addFilterUnit(filterUnit);
            }
            closeTag("fiche");
        }
        closeTag("extraitcorpus");
    }

    private void addExtraitThesaurus(ExtraitThesaurusDef extraitThesaurusDef) throws IOException {
        if (extraitThesaurusDef == null) {
            return;
        }
        startOpenTag("extraitthesaurus");
        addAttribute("extrait-name", extraitThesaurusDef.getExtraitName());
        addTagName(extraitThesaurusDef.getTagName(), ExtractionConstants.EXTRAITTHESAURUS_DEFAULT_TAGNAME);
        endOpenTag();
        for (MotcleQuery motcleQuery : extraitThesaurusDef.getMotcleQueryList()) {
            ExtractionXMLUtils.writeMotcleQuery(this, motcleQuery, "motcle-select");
        }
        addMotcleFilter(extraitThesaurusDef.getMotcleFilter(), ExtractionConstants.MOTCLE_DEFAULT_TAGNAME, null);
        closeTag("extraitthesaurus");
    }

    private void addExtraitAddenda(ExtraitAddendaDef extraitAddendasDef) throws IOException {
        startOpenTag("extraitaddenda");
        addAttribute("extrait-name", extraitAddendasDef.getExtraitName());
        addTagName(extraitAddendasDef.getTagName(), ExtractionConstants.EXTRAITADDENDA_DEFAULT_TAGNAME);
        endOpenTag();
        for (DocumentQuery documentQuery : extraitAddendasDef.getDocumentQueryList()) {
            ExtractionXMLUtils.writeDocumentQuery(this, documentQuery, "document-select");
        }
        DocumentFilter documentFilter = extraitAddendasDef.getDocumentFilter();
        startOpenTag("document");
        addTagName(documentFilter.getTagName(), ExtractionConstants.DOCUMENT_DEFAULT_TAGNAME);
        closeEmptyTag();
        closeTag("extraitaddenda");
    }

    private void addExtraitAlbum(ExtraitAlbumDef extraitAlbumDef) throws IOException {
        startOpenTag("extraitalbum");
        addAttribute("extrait-name", extraitAlbumDef.getExtraitName());
        addTagName(extraitAlbumDef.getTagName(), ExtractionConstants.EXTRAITALBUM_DEFAULT_TAGNAME);
        endOpenTag();
        for (IllustrationQuery illustrationQuery : extraitAlbumDef.getIllustrationQueryList()) {
            ExtractionXMLUtils.writeIllustrationQuery(this, illustrationQuery, "illustration-select");
        }
        IllustrationFilter illustrationFilter = extraitAlbumDef.getIllustrationFilter();
        startOpenTag("illustration");
        addTagName(illustrationFilter.getTagName(), ExtractionConstants.ILLUSTRATION_DEFAULT_TAGNAME);
        closeEmptyTag();
        closeTag("extraitalbum");
    }

    private void addGroupClause(GroupClause groupClause) throws IOException {
        if (groupClause == null) {
            return;
        }
        startOpenTag("group");
        addAttribute("type", ExtractionXMLUtils.typeToString(groupClause.getGroupType()));
        GroupParams groupParams = groupClause.getGroupParams();
        if (groupParams != null) {
            addAttribute("params", groupParams.toString());
        }
        addAttribute("sort", ExtractionXMLUtils.sortToString(groupClause.getSortOrder()));
        addTagName(groupClause.getTagName(), ExtractionConstants.GROUP_DEFAULT_TAGNAME);
        GroupClause subGroupClause = groupClause.getSubGroupClause();
        if (subGroupClause != null) {
            endOpenTag();
            addGroupClause(subGroupClause);
            closeTag("group");
        } else {
            closeEmptyTag();
        }
    }

    private void addPackClause(PackClause packClause) throws IOException {
        if (packClause == null) {
            return;
        }
        startOpenTag("pack");
        addAttribute("size", String.valueOf(packClause.getPackSize()));
        int modulo = packClause.getModulo();
        if (modulo > 0) {
            addAttribute("size", String.valueOf(modulo));
        }
        addTagName(packClause.getTagName(), ExtractionConstants.PACK_DEFAULT_TAGNAME);
        closeEmptyTag();
    }

    private void addIntituleClause(IntituleClause intituleClause) throws IOException {
        if (intituleClause == null) {
            return;
        }
        if (intituleClause.withIntitule()) {
            addEmptyElement("intitule");
        }
    }

    private void addFilterUnit(FilterUnit filterUnit) throws IOException {
        String params = toParamsString(filterUnit.getAttributes());
        if (filterUnit instanceof EnteteFilterUnit) {
            addEmptyElement("entete");
        } else if (filterUnit instanceof CorpsdeficheFilterUnit) {
            addEmptyElement("corpsdefiche");
        } else if (filterUnit instanceof FieldKeyFilterUnit) {
            FieldKeyFilterUnit fieldKeyFilterUnit = (FieldKeyFilterUnit) filterUnit;
            addFieldKey(fieldKeyFilterUnit.getFieldKey(), params);
        } else if (filterUnit instanceof FieldNamePrefixFilterUnit) {
            addFieldNamePrefix((FieldNamePrefixFilterUnit) filterUnit, params);
        } else if (filterUnit instanceof LotFilterUnit) {
            addLot((LotFilterUnit) filterUnit, params);
        } else if (filterUnit instanceof ChronoFilterUnit) {
            addEmptyElement("chrono");
        } else if (filterUnit instanceof IntituleFilterUnit) {
            addEmptyElement("intitule");
        } else if (filterUnit instanceof ExtraitAddendaFilterUnit) {
            addExtraitAddenda(((ExtraitAddendaFilterUnit) filterUnit).getExtraitAddendaDef());
        } else if (filterUnit instanceof ExtraitCorpusFilterUnit) {
            addExtraitCorpus(((ExtraitCorpusFilterUnit) filterUnit).getExtraitCorpusDef());
        } else if (filterUnit instanceof ExtraitThesaurusFilterUnit) {
            addExtraitThesaurus(((ExtraitThesaurusFilterUnit) filterUnit).getExtraitThesaurusDef());
        } else if (filterUnit instanceof ExtraitAlbumFilterUnit) {
            addExtraitAlbum(((ExtraitAlbumFilterUnit) filterUnit).getExtraitAlbumDef());
        } else if (filterUnit instanceof FicheRattachementFilterUnit) {
            addFicheRattachement(((FicheRattachementFilterUnit) filterUnit).getFicheRattachementDef(), params);
        } else if (filterUnit instanceof MotcleRattachementFilterUnit) {
            addMotcleRattachement(((MotcleRattachementFilterUnit) filterUnit).getMotcleRattachementDef(), params);
        }
    }

    private void addTagName(String tagName, String defaultTagName) throws IOException {
        if (tagName == null) {
            addAttribute("tag-name", "NULL");
        } else if (!tagName.equals(defaultTagName)) {
            addAttribute("tag-name", tagName);
        }
    }

    private void addFicheRattachement(FicheRattachementDef ficheRattachementDef, String params) throws IOException {
        FicheFilter ficheFilter = ficheRattachementDef.getFicheFilter();
        startOpenTag("fiche");
        List<SubsetKey> subsetKeyList = ficheRattachementDef.getRattachementSubsetKeyList();
        if (!subsetKeyList.isEmpty()) {
            StringBuilder buf = new StringBuilder();
            for (SubsetKey subsetKey : subsetKeyList) {
                if (buf.length() > 0) {
                    buf.append(',');
                }
                buf.append(subsetKey.getSubsetName());
            }
            addAttribute("corpus", buf.toString());
        }
        addTagName(ficheFilter.getTagName(), ExtractionConstants.FICHE_DEFAULT_TAGNAME);
        addAttribute("params", params);
        List<FilterUnit> filterUnitList = ficheFilter.getFilterUnitList();
        if (filterUnitList.isEmpty()) {
            closeEmptyTag();
        } else {
            endOpenTag();
            for (FilterUnit filterUnit : filterUnitList) {
                addFilterUnit(filterUnit);
            }
            closeTag("fiche");
        }
    }

    private void addMotcleRattachement(MotcleRattachementDef motcleRattachementDef, String params) throws IOException {
        addMotcleFilter(motcleRattachementDef.getMotcleFilter(), ExtractionConstants.MOTCLE_DEFAULT_TAGNAME, params);
    }

    private void addMotcleFilter(MotcleFilter motcleFilter, String tagName, String params) throws IOException {
        startOpenTag(tagName);
        boolean isRecursive = false;
        if (tagName.equals(ExtractionConstants.MOTCLE_DEFAULT_TAGNAME)) {
            MotcleFilter childrenFilter = motcleFilter.getChildrenFilter();
            if (childrenFilter != null) {
                if (!childrenFilter.equals(motcleFilter)) {
                    isRecursive = true;
                }
            }
        } else if (tagName.equals(ExtractionConstants.PARENT_DEFAULT_TAGNAME)) {
            MotcleFilter parentFilter = motcleFilter.getParentFilter();
            if (parentFilter != null) {
                if (!parentFilter.equals(motcleFilter)) {
                    isRecursive = true;
                }
            }
        }
        if (isRecursive) {
            addAttribute("recursive", "1");
        }
        addTagName(motcleFilter.getTagName(), tagName);
        addAttribute("params", params);
        if (motcleFilter.isNoneFiltering()) {
            closeEmptyTag();
            return;
        }
        endOpenTag();
        if (motcleFilter.withLabels()) {
            addEmptyElement("labels");
        }
        if (motcleFilter.withLevel()) {
            addEmptyElement("level");
        }
        if (motcleFilter.withIntitule()) {
            addEmptyElement("intitule");
        }
        MotcleFilter parentFilter = motcleFilter.getParentFilter();
        if (parentFilter != null) {
            if (!parentFilter.equals(motcleFilter)) {
                addMotcleFilter(parentFilter, ExtractionConstants.PARENT_DEFAULT_TAGNAME, null);
            }
        }
        MotcleFilter nextFilter = motcleFilter.getNextFilter();
        if (nextFilter != null) {
            addMotcleFilter(nextFilter, ExtractionConstants.NEXT_DEFAULT_TAGNAME, null);
        }
        MotcleFilter previousFilter = motcleFilter.getPreviousFilter();
        if (previousFilter != null) {
            addMotcleFilter(previousFilter, ExtractionConstants.PREVIOUS_DEFAULT_TAGNAME, null);
        }
        MotcleFilter childrenFilter = motcleFilter.getChildrenFilter();
        if (childrenFilter != null) {
            if (!childrenFilter.equals(motcleFilter)) {
                addMotcleFilter(childrenFilter, ExtractionConstants.MOTCLE_DEFAULT_TAGNAME, null);
            }
        }
        for (FicheRattachementDef ficheRattachementDef : motcleFilter.getFicheRattachementDefList()) {
            addFicheRattachement(ficheRattachementDef, null);
        }
        List<ExtraitCorpusDef> extraitCorpusDefList = motcleFilter.getIndexationExtraitCorpusDefList();
        for (ExtraitCorpusDef extraitCorpusDef : extraitCorpusDefList) {
            addExtraitCorpus(extraitCorpusDef);
        }
        closeTag(tagName);
    }

    private void addFieldKey(FieldKey fieldKey, String params) throws IOException {
        if (fieldKey.isProprieteFieldKey()) {
            startOpenTag("propriete");
            addAttribute("name", fieldKey.getFieldName());
        } else if (fieldKey.isInformationFieldKey()) {
            startOpenTag("information");
            addAttribute("name", fieldKey.getFieldName());
        } else if (fieldKey.isSectionFieldKey()) {
            startOpenTag("section");
            addAttribute("name", fieldKey.getFieldName());
        } else {
            startOpenTag(fieldKey.toString());
        }
        addAttribute("params", params);
        closeEmptyTag();
    }

    private void addFieldNamePrefix(FieldNamePrefixFilterUnit filterUnit, String params) throws IOException {
        short category = filterUnit.getCategory();
        startOpenTag(FieldKey.categoryToString(category));
        addAttribute("name", filterUnit.getPrefix() + "*");
        addAttribute("params", params);
        closeEmptyTag();
    }

    private void addLot(LotFilterUnit lotFilterUnit, String params) throws IOException {
        startOpenTag("lot");
        addTagName(lotFilterUnit.getTagName(), ExtractionConstants.LOT_DEFAULT_TAGNAME);
        addAttribute("params", params);
        endOpenTag();
        for (FilterUnit filterUnit : lotFilterUnit.getFilterUnitList()) {
            addFilterUnit(filterUnit);
        }
        closeTag("lot");
    }

    private String toParamsString(Attributes attributes) {
        if (attributes == null) {
            return null;
        }
        int count = attributes.size();
        if (count == 0) {
            return null;
        }
        StringBuilder buf = new StringBuilder();
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                buf.append(',');
            }
            Attribute attribute = attributes.get(i);
            AttributeKey attributeKey = attribute.getAttributeKey();
            if (attributeKey.getNameSpace().equals(BdfXmlSpace.EXTRACTION_NAMESPACE.toString())) {
                buf.append(attributeKey.getLocalKey());
            } else {
                buf.append(attributeKey.toString());
            }
            buf.append('=');
            buf.append(attribute.getFirstValue());
        }
        return buf.toString();
    }

    public static String toString(ExtractionDef extractionDef, boolean prettyXml, boolean omitXmlDeclaration) {
        StringBuilder buf = new StringBuilder();
        AppendableXMLWriter xmlWriter = XMLUtils.toXMLWriter(buf, (prettyXml) ? 0 : -999);
        try {
            if (!omitXmlDeclaration) {
                xmlWriter.appendXMLDeclaration();
            }
            ExtractionDefXMLPart xmlPart = new ExtractionDefXMLPart(xmlWriter);
            xmlPart.appendExtractionDef(extractionDef);
        } catch (IOException ioe) {
            throw new ShouldNotOccurException(ioe);
        }
        return buf.toString();
    }

}
