/* FichothequeLib_Xml - Copyright (c) 2006-2020 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.xml.extraction;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Set;
import java.util.function.Predicate;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.Corpus;
import net.fichotheque.corpus.FicheMeta;
import net.fichotheque.extraction.ExtractionContext;
import net.fichotheque.extraction.ExtractionSource;
import net.fichotheque.extraction.def.ExtractionDef;
import net.fichotheque.extraction.def.ExtraitCorpusDef;
import net.fichotheque.extraction.def.ExtraitThesaurusDef;
import net.fichotheque.extraction.run.CorpusExtractor;
import net.fichotheque.extraction.run.ExtraitCorpusResult;
import net.fichotheque.extraction.run.ExtraitThesaurusResult;
import net.fichotheque.extraction.run.ThesaurusExtractor;
import net.fichotheque.selection.FicheSelector;
import net.fichotheque.selection.MotcleSelector;
import net.fichotheque.selection.SelectionContext;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.utils.selection.FicheSelectorBuilder;
import net.fichotheque.utils.selection.MotcleSelectorBuilder;
import net.fichotheque.utils.selection.SelectionContextBuilder;
import net.mapeadores.util.exceptions.NestedIOException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.xml.AppendableXMLWriter;
import net.mapeadores.util.xml.DOMUtils;
import net.mapeadores.util.xml.DOMXMLWriter;
import net.mapeadores.util.xml.XMLBuffer;
import net.mapeadores.util.xml.XMLPart;
import net.mapeadores.util.xml.XMLUtils;
import net.mapeadores.util.xml.XMLWriter;
import org.w3c.dom.Document;
import net.fichotheque.selection.FicheQuery;
import net.fichotheque.selection.MotcleQuery;


/**
 *
 * @author Vincent Calame
 */
public class ExtractionXMLPart extends XMLPart {

    private final static SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
    private final ExtractionContext extractionContext;
    private final ExtraitCorpusXMLPart extraitCorpusXMLPart;
    private final ExtraitThesaurusXMLPart extraitThesaurusXMLPart;

    public ExtractionXMLPart(XMLWriter xmlWriter, ExtractionContext extractionContext) {
        super(xmlWriter);
        this.extractionContext = extractionContext;
        extraitCorpusXMLPart = new ExtraitCorpusXMLPart(xmlWriter, extractionContext);
        extraitThesaurusXMLPart = new ExtraitThesaurusXMLPart(xmlWriter, extractionContext);
    }

    public void addExtraction(ExtractionDef extractionDef, Date date, ExtractionSource extractionSource, boolean documentRoot) throws IOException {
        String tagName = extractionDef.getTagName();
        if ((tagName == null) && (documentRoot)) {
            tagName = "NULL";
        }
        if (tagName != null) {
            startOpenTag(tagName);
            addAttribute("date", simpleDateFormat.format(new Date()));
            endOpenTag();
        }
        Object source = extractionSource.getDynamicObject();
        if (source != null) {
            addDynamic(extractionDef, source);
        }
        addStatic(extractionDef.getStaticTagName(), extractionSource);
        if (tagName != null) {
            closeTag(tagName);
        }
    }

    private void addDynamic(ExtractionDef extractionDef, Object source) throws IOException {
        if (source instanceof FicheMeta) {
            addFicheMetaDynamic(extractionDef.getDynamicExtraitCorpusDef(), (FicheMeta) source);
        } else if (source instanceof Corpus) {
            addCorpusDynamic(extractionDef, (Corpus) source);
        } else if (source instanceof Motcle) {
            addMotcleDynamic(extractionDef.getDynamicExtraitThesaurusDef(), (Motcle) source);
        } else if (source instanceof Thesaurus) {
            addThesaurusDynamic(extractionDef.getDynamicExtraitThesaurusDef(), (Thesaurus) source);
        } else {
            throw new ClassCastException("unable to deal with the class " + source.getClass().getName());
        }
    }

    private void addFicheMetaDynamic(ExtraitCorpusDef dynamicExtraitCorpusDef, FicheMeta ficheMeta) throws IOException {
        if (dynamicExtraitCorpusDef == null) {
            return;
        }
        CorpusExtractor corpusExtractor = extractionContext.getExtractorProvider().getCorpusExtractor(dynamicExtraitCorpusDef);
        corpusExtractor.add(ficheMeta, null);
        for (ExtraitCorpusResult extraitCorpusResult : corpusExtractor.getExtraitCorpusResultList()) {
            extraitCorpusXMLPart.addExtraitcorpus(extraitCorpusResult);
        }
    }

    private void addCorpusDynamic(ExtractionDef extractionDef, Corpus corpus) throws IOException {
        ExtraitCorpusDef dynamicExtraitCorpusDef = extractionDef.getDynamicExtraitCorpusDef();
        if (dynamicExtraitCorpusDef == null) {
            return;
        }
        CorpusExtractor corpusExtractor = extractionContext.getExtractorProvider().getCorpusExtractor(dynamicExtraitCorpusDef);
        List<FicheQuery> ficheQueryList = dynamicExtraitCorpusDef.getFicheQueryList();
        if (!ficheQueryList.isEmpty()) {
            SubsetKey corpusKey = corpus.getSubsetKey();
            Set<SubsetKey> corpusKeySingleton = Collections.singleton(corpusKey);
            List<FicheQuery> newQueryList = new ArrayList<FicheQuery>();
            for (FicheQuery query : ficheQueryList) {
                List<SubsetKey> corpusKeyList = query.getCorpusKeyList();
                boolean present = false;
                if (corpusKeyList.isEmpty()) {
                    present = true;
                } else {
                    for (SubsetKey subsetKey : corpusKeyList) {
                        if (subsetKey.equals(corpusKey)) {
                            present = true;
                            break;
                        }
                    }
                }
                if (present) {
                    newQueryList.add(ExtractionXMLUtils.derive(query, corpusKeySingleton));
                }
            }
            if (newQueryList.isEmpty()) {
                return;
            }
            Lang defaultLang = LocalisationUtils.getDefaultLang(extractionContext.getLangContext());
            SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, defaultLang)
                    .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                    .setFichePredicate(extractionContext.getFichePredicate())
                    .toSelectionContext();
            FicheSelector ficheSelector = FicheSelectorBuilder.init(selectionContext).addAll(newQueryList).toFicheSelector();
            for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                if (ficheSelector.test(ficheMeta)) {
                    corpusExtractor.add(ficheMeta, null);
                }
            }
        } else {
            Predicate<FicheMeta> fichePredicate = extractionContext.getFichePredicate();
            for (FicheMeta ficheMeta : corpus.getFicheMetaList()) {
                if ((fichePredicate == null) || (fichePredicate.test(ficheMeta))) {
                    corpusExtractor.add(ficheMeta, null);
                }
            }
        }
        for (ExtraitCorpusResult extraitCorpusResult : corpusExtractor.getExtraitCorpusResultList()) {
            extraitCorpusXMLPart.addExtraitcorpus(extraitCorpusResult);
        }
    }

    private void addMotcleDynamic(ExtraitThesaurusDef dynamicExtraitThesaurusDef, Motcle motcle) throws IOException {
        if (dynamicExtraitThesaurusDef == null) {
            return;
        }
        ThesaurusExtractor thesaurusExtractor = extractionContext.getExtractorProvider().getThesaurusExtractor(dynamicExtraitThesaurusDef);
        thesaurusExtractor.add(motcle, null);
        extraitThesaurusXMLPart.addExtraitThesaurus(thesaurusExtractor.getExtraitThesaurusResult());
    }

    private void addThesaurusDynamic(ExtraitThesaurusDef dynamicExtraitThesaurusDef, Thesaurus thesaurus) throws IOException {
        if (dynamicExtraitThesaurusDef == null) {
            return;
        }
        ThesaurusExtractor thesaurusExtractor = extractionContext.getExtractorProvider().getThesaurusExtractor(dynamicExtraitThesaurusDef);
        List<MotcleQuery> queryList = dynamicExtraitThesaurusDef.getMotcleQueryList();
        if (!queryList.isEmpty()) {
            SubsetKey thesaurusKey = thesaurus.getSubsetKey();
            List<MotcleQuery> newQueryList = new ArrayList<MotcleQuery>();
            for (MotcleQuery query : queryList) {
                boolean here = false;
                for (SubsetKey subsetKey : query.getThesaurusKeyList()) {
                    if (subsetKey.equals(thesaurusKey)) {
                        here = true;
                        break;
                    }
                }
                if (here) {
                    newQueryList.add(ExtractionXMLUtils.derive(query, Collections.singletonList(thesaurusKey)));
                }
            }
            if (newQueryList.isEmpty()) {
                return;
            }
            Lang defaultLang = LocalisationUtils.getDefaultLang(extractionContext.getLangContext());
            SelectionContext selectionContext = SelectionContextBuilder.init(extractionContext, defaultLang)
                    .setSubsetAccessPredicate(extractionContext.getSubsetAccessPredicate())
                    .toSelectionContext();
            MotcleSelector motcleSelector = MotcleSelectorBuilder.init(selectionContext).addAll(queryList).toMotcleSelector();
            for (Motcle motcle : thesaurus.getMotcleList()) {
                if (motcleSelector.test(motcle)) {
                    thesaurusExtractor.add(motcle, null);
                }
            }
        } else {
            for (Motcle motcle : thesaurus.getFirstLevelList()) {
                thesaurusExtractor.add(motcle, null);
            }
        }
        extraitThesaurusXMLPart.addExtraitThesaurus(thesaurusExtractor.getExtraitThesaurusResult());
    }

    private void addStatic(String staticTagName, ExtractionSource extractionSource) throws IOException {
        List<ExtraitCorpusResult> extraitCorpusResultList = extractionSource.getStaticExtraitCorpusResultList();
        List<ExtraitThesaurusResult> extraitThesaurusResultList = extractionSource.getStaticExtraitThesaurusResultList();
        if ((extraitCorpusResultList.isEmpty()) && (extraitThesaurusResultList.isEmpty())) {
            return;
        }
        if (staticTagName != null) {
            openTag(staticTagName);
        }
        for (ExtraitCorpusResult extraitCorpusResult : extraitCorpusResultList) {
            extraitCorpusXMLPart.addExtraitcorpus(extraitCorpusResult);
        }
        for (ExtraitThesaurusResult extraitThesaurusResult : extraitThesaurusResultList) {
            extraitThesaurusXMLPart.addExtraitThesaurus(extraitThesaurusResult);
        }
        if (staticTagName != null) {
            closeTag(staticTagName);
        }
    }

    public static String toString(ExtractionContext extractionContext, ExtractionDef extractionDef, ExtractionSource extractionSource, boolean prettyXml) {
        return toString(extractionContext, extractionDef, extractionSource, prettyXml, false);
    }

    /**
     * Convertit une extraction en chaîne. l'argument <code>prettyXml</code>
     * indique s'il faut mettre tabulations et sauts de ligne.
     */
    public static String toString(ExtractionContext extractionContext, ExtractionDef extractionDef, ExtractionSource extractionSource, boolean prettyXml, boolean omitXmlDeclaration) {
        StringBuilder buf = new StringBuilder(512);
        AppendableXMLWriter xmlWriter = XMLUtils.toXMLWriter(buf, (prettyXml) ? 0 : -999);
        ExtractionXMLPart extractionXMLPart = new ExtractionXMLPart(xmlWriter, extractionContext);
        try {
            if (!omitXmlDeclaration) {
                xmlWriter.appendXMLDeclaration();
            }
            extractionXMLPart.addExtraction(extractionDef, new Date(), extractionSource, true);
        } catch (IOException ioe) {
        }
        return buf.toString();
    }

    public static Document toDOMDocument(ExtractionContext extractionContext, ExtractionDef extractionDef, ExtractionSource extractionSource) {
        Document document = DOMUtils.newDocument();
        DOMXMLWriter xmlWriter = new DOMXMLWriter(document);
        ExtractionXMLPart extractionXMLPart = new ExtractionXMLPart(xmlWriter, extractionContext);
        try {
            extractionXMLPart.addExtraction(extractionDef, new Date(), extractionSource, true);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        return document;
    }

    public static XMLBuffer toXMLBuffer(ExtractionContext extractionContext, ExtractionDef extractionDef, ExtractionSource extractionSource) {
        XMLBuffer xmlBuffer = new XMLBuffer();
        XMLWriter xmlWriter = xmlBuffer.getXMLWriter();
        ExtractionXMLPart extractionXMLPart = new ExtractionXMLPart(xmlWriter, extractionContext);
        try {
            extractionXMLPart.addExtraction(extractionDef, new Date(), extractionSource, true);
        } catch (IOException ioe) {
            throw new NestedIOException(ioe);
        }
        xmlBuffer.flush();
        return xmlBuffer;
    }

}
